#include "Matrix.h"
#include <iostream>
#include <sstream>
#include <cassert>
using namespace std;

int main() {

    // Test matrix operations
    double mdata1[] = 
    { 1, 2,
      3, 4 };
    Matrix m1(2, 2, mdata1);
    cout << m1 << endl;
    double mdata2[] = 
    { 3, 0, 0,
      0, 2, 0,
      0, 0, 1 };
    Matrix m2(3, 3, mdata2);
    assert(m2.trace() == 6);
    
    Matrix m3(m1);
    assert((m1 != m2) && (m1 == m3));
    m2(3, 3) = 3;
    assert(m2(1, 1) == m2(3, 3));
    m3 = m2;
    assert(m2 == m3 && m1 != m3);
    
    double mdata4[] = 
    { 3, 0,
      0, 2,
      2, 1 };
    Matrix m4(3, 2, mdata4);
    double mdata5[] = 
    { 3, 6, 
      6, 8,
      5, 8 };
    Matrix m5(3, 2, mdata5);
    double mdata6[] =
    { 6, 6,
      6, 10,
      7, 9 };
    Matrix m6(3, 2, mdata6);
    
    assert(m5 == m4*m1);
    assert(m5 + m4 == m6);
    assert(m4 + m5 == m6);
    assert(2*m5 == m5*2);
    assert(m5 + m5 == 2*m5);
    assert(5*(m5 + m4) == 5*m5 + 4*m4 + m4);
    Matrix ones(3, 3, 1.0);
    assert(ones*ones == Matrix(3, 3, 3.0));
    assert(Matrix(2, 3, 0.0)*m5 == Matrix(2, 2, 0.0));
    
    Matrix empty;
    assert(empty == m5*empty);
    assert(empty == empty*empty);
    assert(m4*m1*empty == empty*empty);
    assert(empty == Matrix(0, 120));
    
    // Test the Spec procedure
    double mdataa[] = 
     { 3, 0,
       0, 2 };
    Matrix ma(2, 2, mdataa);
    double mdatab[] = 
     { 1, 0,
       0, 1 };
    Matrix mb(2, 2, mdatab);
    
    Matrix three(1, 1, 3.0);
    Matrix two(1, 1, 2.0);
    Matrix zero(1, 1, 0.0);
    Matrix one(1, 1, 1.0);
    
    assert( ((three | zero) & (zero | two)) == ma );
    assert( ma == (ma | empty | empty & empty) );
    assert( (Matrix(100, 100, 3.0) | Matrix(100, 100, 3.0)) == Matrix(100, 200, 3.0));
    assert( (Matrix(100, 100, 3.0) & Matrix(100, 100, 3.0)) == Matrix(200, 100, 3.0));
    
    assert(mb == mb.transpose());
    assert(ma == ma.transpose());
    assert(((three | zero | two) & (zero | two | one)) == m4.transpose());
    assert(m5.transpose().transpose() == m5);
    assert((three | two | one).transpose().transpose() == (three | two | one));
    
    assert(ma.getRowRange(2,2) == (zero | two));
    assert(ma.getRowRange(1,2) == ma);
    assert(m4.getRowRange(1,2) == ma);
    
    assert((m5.transpose().getColRange(1, 1) | m5.transpose().getColRange(2, 3)) ==
           m5.transpose());
    assert(m5.getColRange(1, 2) == m5);
    assert( (m5 | m6).getColRange(3,4) == m6 );
    
    ostringstream out;
    out << m5;
    istringstream in(out.str());
    Matrix read(3, 2, in);
    assert(read == m5);
    
    vector<Matrix::GenEigenvector*> spec = Spec(ma, mb);
    for (int i = 0; i < spec.size(); i++) 
        cout << "Vector: " << endl << spec[i]->eigenvector <<
                "Value:  " << spec[i]->alphar << " + i*" << spec[i]->alphai << 
                " // " << spec[i]->beta << endl;
    
    cout << "Matrix test successful" << endl;
    return 0;
}

