// ------------------------------------------------------------------------------//
// Cross-language text retrieval using kernel RCCA
// (C) 2004  Konstantin Tretyakov, Jelena Zaitseva
//

clear 
K_3mer = read ("K_3mer.dat", 262, 262);

rand('seed', 300);
getf("kernel_methods.sci"); // Load auxiliary subroutines
getf("kernel_crosslang.sci"); 

// Kernel to use
K = K_3mer;

// Extract subkernels
n = size(K, 'r') / 2;
K_x = K(1:n, 1:n);
K_y = K((n+1):(2*n), (n+1):(2*n));

// Split into train (80%) and test (20%) data 
[idx_train, idx_test] = random_split(1:n, 0.8);

// Determine parameters k and reg using 5-fold cross-validation
n_val = floor(length(idx_train)/5);

k_range = [5 10 20 50 80];
reg_range = [1e-3 0.1 0.5 1 10];
clear cv_precisions;
clear test_precisions;
for i_k=1:length(k_range)
	for i_reg=1:length(reg_range)
		k = k_range(i_k);
		reg = reg_range(i_reg);
		printf("Iterating for k=%d and reg=%f...\n", k, reg);
		
		// 5-fold cross validation
		cv_precisions(i_k, i_reg) = 0;
		for i=1:5
			idx_val = idx_train(((i-1)*n_val + 1):(i*n_val));
			idx_trn = idx_train([1:(i-1)*n_val, (i*n_val+1):length(idx_train)]);
  			K_x_train = K_x(idx_trn, idx_trn);
  			K_y_train = K_y(idx_trn, idx_trn);
  			
  			// Train and test model
  			clr_model = clr_train(K_x_train, K_y_train, k, reg);
  			precision = clr_test(clr_model, K_x(idx_trn, idx_val), K_y(idx_trn, idx_val),...
  														  diag(K_x(idx_val, idx_val)), diag(K_y(idx_val, idx_val)), 1);
  			cv_precisions(i_k, i_reg) = cv_precisions(i_k, i_reg) + precision/5;
		end
		
		// Performance on test data
  		K_x_train = K_x(idx_train, idx_train);
  		K_y_train = K_y(idx_train, idx_train);
  		
  		// Train and test model
  		clr_model = clr_train(K_x_train, K_y_train, k, reg);
  		precision = clr_test(clr_model, K_x(idx_train, idx_test), K_y(idx_train, idx_test),...
  													  diag(K_x(idx_test, idx_test)), diag(K_y(idx_test, idx_test)), 1);
  		test_precisions(i_k, i_reg) = precision;
	end
end

[best_cv_prec, which] = max(cv_precisions);
k = k_range(which(1));
reg = reg_range(which(2));
printf("The best cross-validation precision %f was attained at k=%d and reg=%f\n", ...
        best_cv_prec, k, reg);
printf("Corresponding test precision was %f\n", test_precisions(which(1), which(2)));
[best_test_prec, which] = max(test_precisions);
printf("Best test precision %f was attained at k=%d and reg=%f\n", ...
        best_test_prec, k_range(which(1)), reg_range(which(2)));
printf("Corresponding cross-validation precision was %f\n", cv_precisions(which(1), which(2)));

// Calculate how often the correct match is within 5 closest matches
K_x_train = K_x(idx_train, idx_train);
K_y_train = K_y(idx_train, idx_train);
		
// Train and test model
clr_model = clr_train(K_x_train, K_y_train, k, reg);
precision = clr_test(clr_model, K_x(idx_train, idx_test), K_y(idx_train, idx_test),...
											  diag(K_x(idx_test, idx_test)), diag(K_y(idx_test, idx_test)), 5);

printf("Using k=%d and reg=%f the correct match is within 5 best matches in %f cases\n",...
                         k, reg, precision);

// -------------------- EOF -------------------- //


