/*
 * Cross-language text retrieval main model header file
 *
 * Author: Konstantin Tretyakov
 * Created: 21.11.2004
 */

#ifndef __CrosslangModel_h__
#define __CrosslangModel_h__

#include "Matrix.h"
#include <string>
#include <vector>
using std::string;
using std::vector;


// -------------------------------------------------------------------------- //

/**
 * A simple record containing program options
 */
class CrosslangOptions {
public:
    string modelFile; // File to load/save model
    string trainFile; // Training data input file
    string dataFile;  // Database to run on
    string query;     // Query
    string action;    // "train"/"query"
    int kernelType;   // 0 = bow, 1..6 = k-mer
    int k;            // Number of canonical components to use
    double reg;       // Regularization parameter
    int matchCount;   // Number of matches to return for a query
    
    static const string USAGE_MESSAGE;

    CrosslangOptions(int argc, char* argv[]);
};


// -------------------------------------------------------------------------- //

/**
 * Cross-language retrieval model
 */
class CrosslangModel {
protected:
    int n; // Number of training pairs
    Matrix kernelX, kernelXn, kernelXnc; // X kernel, normalized and centered
    Matrix kernelY, kernelYn, kernelYnc; // Y kernel, normalized and centered
    Matrix wX, wY;   // X and Y weight vectors as columns
    vector<string> trainX;
    vector<string> trainY;
    vector<string> data;
    vector<Matrix> dataSemanticVectors;
    CrosslangOptions options;
    bool trained;

public:
    CrosslangModel(const CrosslangOptions& opts);
    
    void train(const vector<string>& trainX, const vector<string>& trainY, int k, double reg);
    void save(const string& filename) const;
    void load(const string& filename);
    void prepareData(const vector<string>& data); 
    Matrix evaluateX(const string& s); // Finds a semantic vector for phrase in lang. X
    Matrix evaluateY(const string& s); // Finds a semantic vector for a phrase in lang. Y
    Matrix evaluateY(const Matrix& k, const string& s); 
    vector<int> findClosestSemanticVectors(const Matrix& q, int k);
};


#endif


