/**
 * Reads given file with a string per line
 * and outputs kernel matrix (of given kernel type)
 * (row per line)
 *
 * Usage: datatokernel <datafile.txt> <k>
 * where type is 
 *    0 for Bag-of-words kernel
 *    k for k-mer kernel (k=1..6)
 *
 * Author: Konstantin Tretyakov
 * Created: 21.11.2004
 */
 
#include "Kernels.h"
#include <iostream>
#include <fstream>
#include <string>
#include <cstdio>
using namespace std;


/**
 * Load strings, calculate kernel matrix, save it.
 */
int main(int argc, char* argv[]) {
    try{
        if (argc < 4) throw string("Usage: datatokernel <datafile.txt> <k> <outfile.mat>");
        
        int kernelToUse = 0; // 0 - bow, N - N-mer
        
        // Determine kernel to use
        if (sscanf(argv[2], "%d", &kernelToUse) == 0 || kernelToUse > 6 || kernelToUse < 0) 
            throw string("Invalid kernel specification. Must be 0 for BOW kernel, 1..6 for K-mer kernel");
        
        // Read strings from file
        vector<string> instances = LoadStrings(argv[1]);
        cout << "Loaded " << instances.size() << " strings." << endl;
        cout << "Calculating kernel..." << endl;
        
        
        Matrix kernelMatrix = EvaluateStringKernel(kernelToUse, instances);
        
        // Write matrix to file
        cout << "Writing matrix to file..." << endl;
        ofstream out(argv[3]);
        out << kernelMatrix;
        out.close();
        
        cout << "Done." << endl;
    }
    catch (string s){
        cout << s << endl;
    }
}

